@@ -129,7 +129,7 @@ t_cached_object *add_file_to_cache(t_session *session, char *file) {
 	struct stat status;
 	off_t size;
 	int fd;
-	size_t bytes_read, bytes_total = 0;
+	ssize_t bytes_read, bytes_total = 0;
 
 	if (file == NULL) {
 		return NULL;
@@ -153,7 +153,7 @@ t_cached_object *add_file_to_cache(t_session *session, char *file) {
 	}
 
 	if ((fd = open(file, O_RDONLY)) != -1) {
-		while (bytes_total < size) {
+		while ((unsigned int)bytes_total < size) {
 			if ((bytes_read = read(fd, object->data + bytes_total, size - bytes_total)) == -1) {
 				if (errno != EINTR) {
 					free(object->data);
@@ -489,7 +489,7 @@ int clear_cache() {
 	return removed;
 }
 
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 void print_cache_list(FILE *fp) {
 	t_cached_object *object;
 	int i, files = 0, secs;
