@@ -50,7 +50,7 @@ void remove_cgi_output_from_cache(t_session *session, char *request_uri);
 void done_with_cached_object(t_cached_object *object, bool remove_object);
 void check_cache(time_t time);
 int clear_cache(void);
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 void print_cache_list(FILE *fp);
 off_t size_of_cache(void);
 #endif
