@@ -178,7 +178,9 @@ bool get_wrap_data(char *wrapid, char *handler, char *cgi, t_wrap *wrap_data, ch
 							item = pipe;
 						}
 
-						wrap_data->cgiroot = strdup(item);
+						if ((wrap_data->cgiroot = strdup(item)) == NULL) {
+							break;
+						}
 					} else if (*item == '~') {
 						if ((pwd = getpwnam(item + 1)) == NULL) {
 							log_error("invalid username");
