@@ -17,11 +17,10 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <pthread.h>
+#include <poll.h>
 #include <sys/socket.h>
 #include <sys/un.h>
-#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
-#endif
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
@@ -456,31 +455,26 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 }
 
 t_cgi_result read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
-	int bytes_read, result = cgi_OKE, highest_fd = -1;
-	struct timeval select_timeout;
-	fd_set read_fds;
+	int bytes_read, result = cgi_OKE;
 	bool read_again;
+	struct pollfd poll_data[2];
+	int poll_size;
+
+	poll_size = 0;
+	if (cgi_info->from_cgi != -1) {
+		poll_data[poll_size].fd = cgi_info->from_cgi;
+		poll_data[poll_size].events = POLL_EVENT_BITS;
+		poll_size++;
+	}
+	if (cgi_info->cgi_error != -1) {
+		poll_data[poll_size].fd = cgi_info->cgi_error;
+		poll_data[poll_size].events = POLL_EVENT_BITS;
+		poll_size++;
+	}
 
 	do {
 		read_again = false;
-
-		FD_ZERO(&read_fds);
-		if (cgi_info->from_cgi != -1) {
-			FD_SET(cgi_info->from_cgi, &read_fds);
-			if (cgi_info->from_cgi > highest_fd) {
-				highest_fd = cgi_info->from_cgi;
-			}
-		}
-		if (cgi_info->cgi_error != -1) {
-			FD_SET(cgi_info->cgi_error, &read_fds);
-			if (cgi_info->cgi_error > highest_fd) {
-				highest_fd = cgi_info->cgi_error;
-			}
-		}
-
-		select_timeout.tv_sec = 1;
-		select_timeout.tv_usec = 0;
-		switch (select(highest_fd + 1, &read_fds, NULL, NULL, &select_timeout)) {
+		switch (poll((struct pollfd*)&poll_data, poll_size, 1000)) {
 			case -1:
 				if (errno != EINTR) {
 					return cgi_ERROR;
@@ -501,7 +495,7 @@ t_cgi_result read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 	/* Read CGI output
 	 */
 	if (cgi_info->from_cgi != -1) {
-		if (FD_ISSET(cgi_info->from_cgi, &read_fds)) do {
+		if (poll_data[0].revents != 0) do {
 			read_again = false;
 
 			bytes_read = read(cgi_info->from_cgi, cgi_info->input_buffer + cgi_info->input_len, cgi_info->input_buffer_size - cgi_info->input_len);
@@ -525,25 +519,30 @@ t_cgi_result read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 	/* Read CGI error output
 	 */
 	if (cgi_info->cgi_error != -1) {
-		if (FD_ISSET(cgi_info->cgi_error, &read_fds)) do {
-			read_again = false;
+		if (poll_data[1].revents != 0) {
+			do {
+				read_again = false;
 
-			bytes_read = read(cgi_info->cgi_error, cgi_info->error_buffer + cgi_info->error_len, cgi_info->error_buffer_size - cgi_info->error_len);
-			switch (bytes_read) {
-				case -1:
-					if (errno != EINTR) {
-						return cgi_ERROR;
-					}
-					read_again = true;
-					break;
-				case 0:
-					close(cgi_info->cgi_error);
-					cgi_info->cgi_error = -1;
-					break;
-				default:
-					cgi_info->error_len += bytes_read;
-			}
-		} while (read_again);
+				bytes_read = read(cgi_info->cgi_error, cgi_info->error_buffer + cgi_info->error_len, cgi_info->error_buffer_size - cgi_info->error_len);
+				switch (bytes_read) {
+					case -1:
+						if (errno != EINTR) {
+							return cgi_ERROR;
+						}
+						read_again = true;
+						break;
+					case 0:
+						close(cgi_info->cgi_error);
+						cgi_info->cgi_error = -1;
+						break;
+					default:
+						cgi_info->error_len += bytes_read;
+				}
+			} while (read_again);
+		} else if (recv(cgi_info->cgi_error, cgi_info->error_buffer + cgi_info->error_len, cgi_info->error_buffer_size - cgi_info->error_len, MSG_PEEK | MSG_DONTWAIT) <= 0) {
+			close(cgi_info->cgi_error);
+			cgi_info->cgi_error = -1;
+		}
 	}
 
 	if ((cgi_info->from_cgi == -1) && (cgi_info->cgi_error == -1)) {
@@ -667,19 +666,15 @@ int send_fcgi_request(t_session *session, int sock) {
 
 static t_cgi_result read_fcgi_socket(t_session *session, t_cgi_info *cgi_info, char *buffer, int size) {
 	int bytes_read;
-	struct timeval select_timeout;
-	fd_set read_fds;
 	bool read_again;
+	struct pollfd poll_data;
+
+	poll_data.fd = cgi_info->from_cgi;
+	poll_data.events = POLL_EVENT_BITS;
 
 	do {
 		read_again = false;
-
-		FD_ZERO(&read_fds);
-		FD_SET(cgi_info->from_cgi, &read_fds);
-
-		select_timeout.tv_sec = 1;
-		select_timeout.tv_usec = 0;
-		switch (select(cgi_info->from_cgi + 1, &read_fds, NULL, NULL, &select_timeout)) {
+		switch (poll(&poll_data, 1, 1000)) {
 			case -1:
 				if (errno == EINTR) {
 					read_again = true;
@@ -778,7 +773,7 @@ t_cgi_result read_from_fcgi_server(t_session *session, t_cgi_info *cgi_info) {
 
 	/* Read data
 	 */
-	if (bytes_left > (data_length - cgi_info->fcgi_data_len)) {
+	if ((unsigned int)bytes_left > (data_length - cgi_info->fcgi_data_len)) {
 		bytes_left = data_length - cgi_info->fcgi_data_len;
 	}
 
