@@ -19,8 +19,8 @@
 #include <sys/wait.h>
 #include "global.h"
 #include "client.h"
-#ifdef ENABLE_COMMAND
-#include "command.h"
+#ifdef ENABLE_TOMAHAWK
+#include "tomahawk.h"
 #endif
 #ifdef ENABLE_MONITOR
 #include "monitor.h"
@@ -441,7 +441,7 @@ int ban_ip(t_ip_addr *ip, int timer, bool kick_on_ban) {
 			ban->next = banlist;
 			banlist = ban;
 
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 			increment_counter(COUNTER_BAN);
 #endif
 
@@ -654,7 +654,7 @@ void close_client_sockets_for_cgi_run(void) {
 	}
 }
 
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 /* Print the list of current connections.
  */
 void print_client_list(FILE *fp) {
