@@ -120,13 +120,12 @@ void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
 #endif
 
 	add_to_environment(fcgi_buffer, "GATEWAY_INTERFACE", "CGI/1.1");
-
 	add_to_environment(fcgi_buffer, "REQUEST_METHOD", session->method);
 	add_to_environment(fcgi_buffer, "REQUEST_URI", session->request_uri);
 
 	if (session->path_info != NULL) {
 		path_info_len = strlen(session->path_info);
-		if (path_info_len <= session->uri_len) {
+		if (path_info_len <= (unsigned int)session->uri_len) {
 			if (memcmp(session->uri + session->uri_len - path_info_len, session->path_info, path_info_len) == 0) {
 				has_path_info = true;
 			}
@@ -234,7 +233,7 @@ void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
 	 */
 	headerfields = session->headerfields;
 	while (headerfields != NULL) {
-		if (strncmp(headerfields->data, "X-", 2) == 0) {
+		if (strncasecmp(headerfields->data, "X-", 2) == 0) {
 			if (header_to_variable(headerfields->data, (char*)&variable, MAX_HEADER_LEN) != -1) {
 				add_to_environment(fcgi_buffer, variable, headerfields->data + headerfields->value_offset);
 			}
