@@ -17,6 +17,7 @@
 #include <stdbool.h>
 #include <unistd.h>
 #include <string.h>
+#include <poll.h>
 #include <fcntl.h>
 #include <time.h>
 #include <signal.h>
@@ -34,6 +35,7 @@
 #include <sys/wait.h>
 #include <sys/socket.h>
 #include <sys/time.h>
+#include "global.h"
 #include "alternative.h"
 #include "mimetype.h"
 #include "serverconfig.h"
@@ -51,8 +53,8 @@
 #include "log.h"
 #include "envir.h"
 #include "global.h"
-#ifdef ENABLE_COMMAND
-#include "command.h"
+#ifdef ENABLE_TOMAHAWK
+#include "tomahawk.h"
 #endif
 #ifdef ENABLE_SSL
 #include "libssl.h"
@@ -77,8 +79,8 @@
 #define rs_CLEAR_CACHE           4
 
 #define PTHREAD_STACK_SIZE 512 * KILOBYTE
-
 #define LOG_PERM (S_IRUSR|S_IWUSR|S_IRGRP)
+#define MAX_ADMIN_CONNECTIONS 3
 
 typedef struct {
 	char *config_dir;
@@ -102,16 +104,12 @@ char *fb_symlink     = "symlink not allowed";
 char *fb_accesslist  = "access denied via accesslist";
 char *fb_alterlist   = "access denied via alterlist";
 char *version_string = "Hiawatha v"VERSION
-
 #ifdef ENABLE_CACHE
 	", cache"
 #endif
 #ifdef ENABLE_CHROOT
 	", chroot"
 #endif
-#ifdef ENABLE_COMMAND
-	", Command Channel"
-#endif
 #ifdef ENABLE_DEBUG
 	", debug"
 #endif
@@ -127,6 +125,9 @@ char *version_string = "Hiawatha v"VERSION
 #ifdef ENABLE_SSL
 	", SSL"
 #endif
+#ifdef ENABLE_TOMAHAWK
+	", Tomahawk"
+#endif
 #ifdef ENABLE_TOOLKIT
 	", URL toolkit"
 #endif
@@ -373,7 +374,7 @@ t_access allow_client(t_session *session) {
 /* Serve the client that connected to the webserver
  */
 int serve_client(t_session *session) {
-	int result, length;
+	int result, length, auth_result;
 	char *search, *qmark, chr, *client_ip;
 	t_host *host_record;
 	t_access access;
@@ -427,7 +428,7 @@ int serve_client(t_session *session) {
 
 		if ((host_record = get_hostrecord(session->config->first_host, session->hostname, session->binding)) != NULL) {
 			session->host = host_record;
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 			session->last_host = host_record;
 #endif
 		}
@@ -475,7 +476,7 @@ int serve_client(t_session *session) {
 				}
 
 				log_exploit_attempt(session, "denied body", session->body);
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 				increment_counter(COUNTER_EXPLOIT);
 #endif
 #ifdef ENABLE_MONITOR
@@ -537,8 +538,8 @@ int serve_client(t_session *session) {
 					break;
 				case pwd:
 				case unspecified:
-					if (http_authentication_oke(session, access == unspecified) == false) {
-						return 401;
+					if ((auth_result = http_authentication_result(session, access == unspecified)) != 200) {
+						return auth_result;
 					}
 			}
 
@@ -596,7 +597,7 @@ int serve_client(t_session *session) {
 	                     session->binding->use_ssl, session->host->allow_dot_files, session->headerfields);
 #else
 	init_toolkit_options(&toolkit_options, session->host->website_root, session->config->url_toolkit,
-	                     false, session->host->allow_dot_files, session->headerfields);
+	                     session->host->allow_dot_files, session->headerfields);
 #endif
 
 	if ((session->request_method != PUT) && (session->request_method != DELETE)) {
@@ -626,7 +627,9 @@ int serve_client(t_session *session) {
 			}
 
 			if (result == UT_REDIRECT) {
-				session->location = strdup(toolkit_options.new_url);
+				if ((session->location = strdup(toolkit_options.new_url)) == NULL) {
+					return -1;
+				}
 				session->cause_of_301 = location;
 				return 301;
 			}
@@ -689,8 +692,8 @@ int serve_client(t_session *session) {
 			break;
 		case pwd:
 		case unspecified:
-			if (http_authentication_oke(session, access == unspecified) == false) {
-				return 401;
+			if ((auth_result = http_authentication_result(session, access == unspecified)) != 200) {
+				return auth_result;
 			}
 	}
 
@@ -1128,8 +1131,8 @@ void task_runner(t_config *config) {
 			delay++;
 		}
 
-#ifdef ENABLE_COMMAND
-		/* CommandChannel check
+#ifdef ENABLE_TOMAHAWK
+		/* Tomahawk check
 		 */
 		check_admin_list();
 #endif
@@ -1203,70 +1206,29 @@ void task_runner(t_config *config) {
 
 /* Signal handlers
  */
-void SEGV_handler(int sig) {
+void SEGV_handler() {
 	syslog(LOG_DAEMON | LOG_ALERT, "segmentation fault!");
 	exit(EXIT_FAILURE);
 }
 
-void TERM_handler(int sig) {
+void TERM_handler() {
 	received_signal = rs_QUIT_SERVER;
 }
 
-void HUP_handler(int sig) {
+void HUP_handler() {
 	received_signal = rs_UNLOCK_LOGFILES;
 }
 
-void USR1_handler(int sig) {
+void USR1_handler() {
 	received_signal = rs_UNBAN_CLIENTS;
 }
 
 #ifdef ENABLE_CACHE
-void USR2_handler(int sig) {
+void USR2_handler() {
 	received_signal = rs_CLEAR_CACHE;
 }
 #endif
 
-/* Fill a filedescriptor set with sockets.
- */
-#ifndef ENABLE_COMMAND
-int fill_read_fds(fd_set *read_fds, t_binding *binding) {
-#else
-int fill_read_fds(fd_set *read_fds, t_binding *binding, t_binding *bind_command) {
-	t_admin *admin;
-#endif
-	int highest_fd = 0;
-
-	FD_ZERO(read_fds);
-	while (binding != NULL) {
-		FD_SET(binding->socket, read_fds);
-		if (binding->socket > highest_fd) {
-			highest_fd = binding->socket;
-		}
-		binding = binding->next;
-	}
-
-#ifdef ENABLE_COMMAND
-	while (bind_command != NULL) {
-		FD_SET(bind_command->socket, read_fds);
-		if (bind_command->socket > highest_fd) {
-			highest_fd = bind_command->socket;
-		}
-		bind_command = bind_command->next;
-	}
-
-	admin = first_admin();
-	while (admin != NULL) {
-		FD_SET(admin->socket, read_fds);
-		if (admin->socket > highest_fd) {
-			highest_fd = admin->socket;
-		}
-		admin = next_admin();
-	}
-#endif
-
-	return highest_fd;
-}
-
 /* Create a socketlist.
  */
 int bind_sockets(t_binding *binding) {
@@ -1498,7 +1460,7 @@ int accept_connection(t_binding *binding, t_config *config) {
 			if (config->reban_during_ban && (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny)) {
 				reban_ip(&(session->ip_address));
 			}
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 			increment_counter(COUNTER_DENY);
 #endif
 			break;
@@ -1515,20 +1477,21 @@ int accept_connection(t_binding *binding, t_config *config) {
 /* Run the Hiawatha webserver.
  */
 int run_server(t_settings *settings) {
-	int                highest_fd;
-	struct timeval     select_timeout;
+	int                number_of_bindings;
 	pthread_attr_t     task_runner_attr;
 	pthread_t          task_runner_thread;
-#ifdef ENABLE_COMMAND
+	struct pollfd      *poll_data, *current_poll;
+#ifdef ENABLE_TOMAHAWK
+	int                number_of_admins;
 	t_admin            *admin;
 	struct sockaddr_in caddr;
 	socklen_t          size;
 	int                admin_socket;
+	FILE               *admin_fp;
 #endif
 	pid_t              pid;
 	t_binding          *binding;
 	t_config           *config;
-    fd_set             read_fds;
 #ifndef CYGWIN
 	struct stat        status;
 	mode_t             access_rights;
@@ -1588,10 +1551,10 @@ int run_server(t_settings *settings) {
 	}
 #endif
 
-#ifdef ENABLE_COMMAND
-	/* Bind CommandChannels
+#ifdef ENABLE_TOMAHAWK
+	/* Bind Tomahawk
 	 */
-	if (bind_sockets(config->command_port) == -1) {
+	if (bind_sockets(config->tomahawk_port) == -1) {
 		return -1;
 	}
 #endif
@@ -1780,25 +1743,26 @@ int run_server(t_settings *settings) {
 		}
 		binding = binding->next;
 	}
-#ifdef ENABLE_COMMAND
-	binding = config->command_port;
+#ifdef ENABLE_TOMAHAWK
+	binding = config->tomahawk_port;
 	while (binding != NULL) {
 		if (listen(binding->socket, 1) == -1) {
-			perror("listen(commandchannel)");
+			perror("listen(tomahawk)");
 			return -1;
 		}
 		binding = binding->next;
 	}
 #endif
 
+	init_send_module();
 	init_log_module();
 	init_client_module();
 	init_load_balancer(config->fcgi_server);
 #ifdef ENABLE_CACHE
 	init_cache_module();
 #endif
-#ifdef ENABLE_COMMAND
-	init_command_module();
+#ifdef ENABLE_TOMAHAWK
+	init_tomahawk_module();
 #endif
 #ifdef ENABLE_XSLT
 	init_xslt_module();
@@ -1862,18 +1826,75 @@ int run_server(t_settings *settings) {
 	}
 	pthread_attr_destroy(&task_runner_attr);
 
+	/* Count bindings
+	 */
+	number_of_bindings = 0;
+
+	binding = config->binding;
+	while (binding != NULL) {
+		number_of_bindings++;
+		binding = binding->next;
+	}
+
+#ifdef ENABLE_TOMAHAWK
+	binding = config->tomahawk_port;
+	while (binding != NULL) {
+		number_of_bindings++;
+		binding = binding->next;
+	}
+#endif
+
+	/* Setup poll data
+	 */
+	if ((poll_data = (struct pollfd*)malloc((number_of_bindings + MAX_ADMIN_CONNECTIONS) * sizeof(struct pollfd*))) == NULL) {
+		return -1;
+	}
+
+	current_poll = poll_data;
+
+	binding = config->binding;
+	while (binding != NULL) {
+		current_poll->fd = binding->socket;
+		current_poll->events = POLL_EVENT_BITS;
+		binding->poll_data = current_poll;
+
+		current_poll++;
+		binding = binding->next;
+	}
+
+#ifdef ENABLE_TOMAHAWK
+	binding = config->tomahawk_port;
+	while (binding != NULL) {
+		current_poll->fd = binding->socket;
+		current_poll->events = POLL_EVENT_BITS;
+		binding->poll_data = current_poll;
+
+		current_poll++;
+		binding = binding->next;
+	}
+#endif
+
 	/* Main loop
 	 */
 	do {
-#ifdef ENABLE_COMMAND
-		highest_fd = fill_read_fds(&read_fds, config->binding, config->command_port);
-#else
-		highest_fd = fill_read_fds(&read_fds, config->binding);
-#endif
+#ifdef ENABLE_TOMAHAWK
+		current_poll = poll_data + number_of_bindings;
+		number_of_admins = 0;
+		admin = first_admin();
+		while (admin != NULL) {
+			current_poll->fd = admin->socket;
+			current_poll->events = POLL_EVENT_BITS;
+			admin->poll_data = current_poll;
 
-		select_timeout.tv_sec = 1;
-		select_timeout.tv_usec = 0;
-		switch (select(highest_fd + 1, &read_fds, NULL, NULL, &select_timeout)) {
+			number_of_admins++;
+			current_poll++;
+			admin = next_admin();
+		}
+
+		switch (poll(poll_data, number_of_bindings + number_of_admins, 1000)) {
+#else
+		switch (poll(poll_data, number_of_bindings, 1000)) {
+#endif
 			case -1:
 				if (errno != EINTR) {
 					log_string(config->system_logfile, "Fatal error selecting connection");
@@ -1883,11 +1904,11 @@ int run_server(t_settings *settings) {
 			case 0:
 				break;
 			default:
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 				/* Connected admins */
 				admin = first_admin();
 				while (admin != NULL) {
-					if (FD_ISSET(admin->socket, &read_fds)) {
+					if (admin->poll_data->revents != 0) {
 						if (handle_admin(admin, config) == cc_DISCONNECT) {
 							remove_admin(admin->socket);
 						}
@@ -1899,7 +1920,7 @@ int run_server(t_settings *settings) {
 				/* HTTP(S) ports */
 				binding = config->binding;
 				while (binding != NULL) {
-					if (FD_ISSET(binding->socket, &read_fds)) {
+					if (binding->poll_data->revents != 0) {
 						if (accept_connection(binding, config) != 0) {
 							usleep(1000);
 							break;
@@ -1909,19 +1930,24 @@ int run_server(t_settings *settings) {
 					binding = binding->next;
 				}
 
-#ifdef ENABLE_COMMAND
-				/* Command ports */
-				binding = config->command_port;
+#ifdef ENABLE_TOMAHAWK
+				/* Tomahawk ports */
+				binding = config->tomahawk_port;
 				while (binding != NULL) {
-					if (FD_ISSET(binding->socket, &read_fds)) {
+					if (binding->poll_data->revents != 0) {
 						size = sizeof(struct sockaddr_in);
 						memset((void*)&caddr, 0, (size_t)size);
 						if ((admin_socket = accept(binding->socket, (struct sockaddr*)&caddr, &size)) == -1) {
 							if (errno != EINTR) {
-								log_string(config->system_logfile, "Fatal error accepting CommandChannel connection");
+								log_string(config->system_logfile, "Fatal error accepting Tomahawk connection");
 								usleep(1000);
 								break;
 							}
+						} else if (number_of_admins >= MAX_ADMIN_CONNECTIONS) {
+							if ((admin_fp = fdopen(admin_socket, "r+")) != NULL) {
+								fprintf(admin_fp, "Maximum number of admin connections reached.\n\n");
+							}
+							fclose(admin_fp);
 						} else if (add_admin(admin_socket) == -1) {
 							close(admin_socket);
 						}
@@ -1937,12 +1963,12 @@ int run_server(t_settings *settings) {
 	close_bindings(config->binding);
 
 	disconnect_clients(config);
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 	disconnect_admins();
 #endif
 
-#ifdef ENABLE_COMMAND
-	binding = config->command_port;
+#ifdef ENABLE_TOMAHAWK
+	binding = config->tomahawk_port;
 	while (binding != NULL) {
 		close(binding->socket);
 		binding = binding->next;
