@@ -17,10 +17,12 @@
 #include <string.h>
 #include <limits.h>
 #include <errno.h>
+#include <poll.h>
 #include <sys/time.h>
+#include "global.h"
 #include "session.h"
 #include "libstr.h"
-#include "command.h"
+#include "tomahawk.h"
 #include "log.h"
 #ifdef ENABLE_MONITOR
 #include "monitor.h"
@@ -37,10 +39,9 @@ extern char *hs_conlen;
 int fetch_request(t_session *session) {
 	char *new_reqbuf, *strstart, *strend;
 	long max_request_size, bytes_read, header_length = -1, content_length = -1;
-	int result = 200, write_bytes, select_result;
+	int result = 200, write_bytes, poll_result;
 	time_t deadline;
-	fd_set read_fds;
-	struct timeval select_timeout;
+	struct pollfd poll_data;
 	bool keep_reading = true, store_on_disk = false;
 	int upload_handle = -1;
 
@@ -107,6 +108,8 @@ int fetch_request(t_session *session) {
 							}
 
 							if (store_on_disk) {
+								/* Write to file on disk
+								 */
 								session->content_length = 0;
 								if (content_length > session->binding->max_upload_size) {
 									result = 413;
@@ -122,6 +125,8 @@ int fetch_request(t_session *session) {
 									break;
 								}
 							} else {
+								/* Read into memory
+								 */
 								session->content_length = content_length;
 								if (header_length + content_length > max_request_size) {
 									session->error_cause = ec_MAX_REQUESTSIZE;
@@ -166,20 +171,19 @@ int fetch_request(t_session *session) {
 			}
 		}
 
-		FD_ZERO(&read_fds);
-		FD_SET(session->client_socket, &read_fds);
-
-		select_timeout.tv_sec = 1;
-		select_timeout.tv_usec = 0;
-
 #ifdef ENABLE_SSL
-		select_result = session->binding->use_ssl ? ssl_pending(&(session->ssl_data)) : 0;
+		poll_result = session->binding->use_ssl ? ssl_pending(&(session->ssl_data)) : 0;
 
-		if (select_result == 0)
+		if (poll_result == 0) {
+#endif
+			poll_data.fd = session->client_socket;
+			poll_data.events = POLL_EVENT_BITS;
+			poll_result = poll(&poll_data, 1, 1000);
+#ifdef ENABLE_SSL
+		}
 #endif
-			select_result = select(session->client_socket + 1, &read_fds, NULL, NULL, &select_timeout);
 
-		switch (select_result) {
+		switch (poll_result) {
 			case -1:
 				if (errno != EINTR) {
 					session->error_cause = ec_SOCKET_READ_ERROR;
@@ -249,8 +253,8 @@ int fetch_request(t_session *session) {
 								break;
 							}
 							if ((session->uploaded_size += write_bytes) > session->binding->max_upload_size) {
-								result = 413;
 								keep_reading = false;
+								result = 413;
 								break;
 							}
 							if (write_bytes < bytes_read) {
@@ -280,7 +284,7 @@ int fetch_request(t_session *session) {
 		close(upload_handle);
 	}
 
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 	increment_transfer(TRANSFER_RECEIVED, header_length + content_length);
 #endif
 
@@ -528,7 +532,7 @@ bool validate_url(t_session *session) {
 	}
 
 	log_exploit_attempt(session, "invalid URL", NULL);
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 	increment_counter(COUNTER_EXPLOIT);
 #endif
 #ifdef ENABLE_MONITOR
