@@ -29,6 +29,11 @@
 #include "log.h"
 #include "polarssl/md5.h"
 
+#define ha_ALLOWED   200
+#define ha_DENIED    401
+#define ha_FORBIDDEN 403
+#define ha_ERROR     500
+
 /* If a required_group_file exist, is a user in the right group?
  */
 bool group_oke(t_session *session, char *user, t_charlist *group) {
@@ -197,46 +202,66 @@ static char *get_A1(t_session *session, char *username, char *realm) {
 
 /* Basic HTTP authentication.
  */
-static bool basic_http_authentication(t_session *session, char *auth_str) {
-	bool retval = false;
+static int basic_http_authentication(t_session *session, char *auth_str) {
+	int retval;
 	char *auth_user, *auth_passwd, *passwd, *encrypted, salt[3];
 
 	if ((auth_user = strdup(auth_str)) == NULL) {
-		return false;
+		return ha_ERROR;
 	}
 
-	if (decode_base64(auth_user)) {
-		auth_passwd = auth_user;
-		while ((*auth_passwd != ':') && (*auth_passwd != '\0')) {
-			auth_passwd++;
-		}
-		if (*auth_passwd == ':') {
-			*(auth_passwd++) = '\0';
-
-			if ((passwd = get_password(session, auth_user)) != NULL) {
-				if (group_oke(session, auth_user, &(session->host->required_group))) {
-					salt[0] = *passwd;
-					salt[1] = *(passwd + 1);
-					salt[2] = '\0';
-					encrypted = crypt(auth_passwd, salt);
-
-					/* Password match?
-					 */
-					if (strcmp(encrypted, passwd) == 0) {
-						retval = ((session->remote_user = strdup(auth_user)) != NULL);
-					} else {
-						register_wrong_password(session);
-					}
-				} else {
-					register_wrong_password(session);
-				}
-				free(passwd);
-			} else  {
-				register_wrong_password(session);
-			}
-		}
+	/* Decode authentication string
+	 */
+	if (decode_base64(auth_user) == false) {
+		register_wrong_password(session);
+		free(auth_user);
+		return ha_DENIED;
 	}
+
+	/* Search for password
+	 */
+	auth_passwd = auth_user;
+	while ((*auth_passwd != ':') && (*auth_passwd != '\0')) {
+		auth_passwd++;
+	}
+	if (*auth_passwd != ':') {
+		register_wrong_password(session);
+		free(auth_user);
+		return ha_DENIED;
+	}
+	*(auth_passwd++) = '\0';
+
+	/* Group oke?
+	 */
+	if (group_oke(session, auth_user, &(session->host->required_group)) == false) {
+		free(auth_user);
+		return ha_FORBIDDEN;
+	}
+
+	/* Read password from file
+	 */
+	if ((passwd = get_password(session, auth_user)) == NULL) {
+		register_wrong_password(session);
+		free(auth_user);
+		return ha_DENIED;
+	}
+
+	salt[0] = *passwd;
+	salt[1] = *(passwd + 1);
+	salt[2] = '\0';
+	encrypted = crypt(auth_passwd, salt);
+
+	/* Password match?
+	 */
+	if (strcmp(encrypted, passwd) == 0) {
+		retval = ((session->remote_user = strdup(auth_user)) != NULL) ? ha_ALLOWED : ha_ERROR;
+	} else {
+		register_wrong_password(session);
+		retval = ha_DENIED;
+	}
+
 	free(auth_user);
+	free(passwd);
 
 	return retval;
 }
@@ -262,7 +287,7 @@ static char *unquoted(char *str) {
 
 /* Digest HTTP authentication.
  */
-static bool digest_http_authentication(t_session *session, char *auth_str) {
+static int digest_http_authentication(t_session *session, char *auth_str) {
 	bool quote_found;
 	unsigned char digest[16];
 	char *key, *value, *rest, *empty = "", *passwd, A1[33], A2[33], result[33];
@@ -283,7 +308,7 @@ static bool digest_http_authentication(t_session *session, char *auth_str) {
 						rest++;
 						break;
 					} else {
-						return false;
+						return ha_DENIED;
 					}
 				}
 				quote_found = true;
@@ -322,33 +347,34 @@ static bool digest_http_authentication(t_session *session, char *auth_str) {
 		key = rest;
 	}
 
+	/* Correct URI?
+	 */
 	if (strcmp(session->request_uri, uri) != 0) {
-		return false;
+		return ha_DENIED;
+	}
+
+	/* Group oke?
+	 */
+	if (group_oke(session, username, &(session->host->required_group)) == false) {
+		return ha_FORBIDDEN;
 	}
 
 	/* Retrieve A1 from passwordfile
 	 */
 	if ((passwd = get_A1(session, username, realm)) == NULL) {
 		register_wrong_password(session);
-		return false;
+		return ha_DENIED;
 	} else if (strlen(passwd) != 32) {
 		free(passwd);
-		return false;
+		return ha_DENIED;
 	}
 	memcpy(A1, passwd, 33);
 	free(passwd);
 
-	/* Group oke?
-	 */
-	if (group_oke(session, username, &(session->host->required_group)) == false) {
-		register_wrong_password(session);
-		return false;
-	}
-
 	/* Calculate A2
 	 */
 	if ((value = (char*)malloc(strlen(session->method) + strlen(uri) + 2)) == NULL) {
-		return false;
+		return ha_ERROR;
 	}
 	sprintf(value, "%s:%s", session->method, uri);
 	md5((unsigned char*)value, strlen(value), digest);
@@ -358,7 +384,7 @@ static bool digest_http_authentication(t_session *session, char *auth_str) {
 	/* Calculate response
 	 */
 	if ((value = (char*)malloc(strlen(A1) + strlen(nonce) + strlen(A2) + 6)) == NULL) {
-		return false;
+		return ha_ERROR;
 	}
 	sprintf(value, "%s:%s:%s", A1, nonce, A2);
 	md5((unsigned char*)value, strlen(value), digest);
@@ -369,22 +395,24 @@ static bool digest_http_authentication(t_session *session, char *auth_str) {
 	 */
 	if (strcmp(result, response) != 0) {
 		register_wrong_password(session);
-		return false;
+		return ha_DENIED;
 	}
 
-	return (session->remote_user = strdup(username)) != NULL;
+	return ((session->remote_user = strdup(username)) != NULL) ? ha_ALLOWED : ha_ERROR;
 }
 
 /* Check if the file is protected by an .hiawatha file with passwordfile setting.
  */
-bool http_authentication_oke(t_session *session, bool on_pwdfile_missing) {
+int http_authentication_result(t_session *session, bool access_on_pwdfile_missing) {
 	char *auth_str;
-	bool result = false;
+	int result = ha_DENIED;
 
 	if (session->host->passwordfile == NULL) {
-		return on_pwdfile_missing;
+		return access_on_pwdfile_missing ? ha_ALLOWED : ha_DENIED;
 	} else if ((auth_str = get_headerfield("Authorization:", session->headerfields)) != NULL) {
-		auth_str = strdup(auth_str);
+		if ((auth_str = strdup(auth_str)) == NULL) {
+			return ha_ERROR;
+		}
 		if ((strncmp(auth_str, "Basic ", 6) == 0) && (session->host->auth_method == basic)) {
 			session->http_auth = basic;
 			result = basic_http_authentication(session, auth_str + 6);
