@@ -138,7 +138,7 @@ void init_charlist(t_charlist *list) {
 }
 
 int parse_charlist(char *value, t_charlist *list) {
-	char *scan;
+	char *scan, **new;
 	int add = 1, i;
 
 	if ((value == NULL) || (list == NULL)) {
@@ -154,9 +154,10 @@ int parse_charlist(char *value, t_charlist *list) {
 		scan++;
 	}
 
-	if ((list->item = (char**)realloc(list->item, (list->size + add) * sizeof(char*))) == NULL) {
+	if ((new = (char**)realloc(list->item, (list->size + add) * sizeof(char*))) == NULL) {
 		return -1;
 	}
+	list->item = new;
 
 	for (i = 0; i < add; i++) {
 		*(list->item + list->size + i) = NULL;
@@ -294,13 +295,13 @@ t_accesslist *parse_accesslist(char *line, bool pwd_allowed, t_accesslist *list)
 					/* Check netmask
 					 */
 					if (new->ip.family == AF_INET) {
-						if (new->netmask > 8 * IPv4_LEN) {
+						if ((unsigned int)new->netmask > 8 * IPv4_LEN) {
 							error = true;
 							break;
 						}
 #ifdef ENABLE_IPV6
 					} else if (new->ip.family == AF_INET6) {
-						if (new->netmask > 8 * IPv6_LEN) {
+						if ((unsigned int)new->netmask > 8 * IPv6_LEN) {
 							error = true;
 							break;
 						}
