@@ -519,7 +519,7 @@ int add_str(char **buffer, int *size, int extra_size, int *len, char *str) {
 	char *new;
 
 	str_len = strlen(str);
-	while (*len + str_len >= *size) {
+	while (*len + (int)str_len >= *size) {
 		*size += extra_size;
 		if ((new = (char*)realloc(*buffer, *size)) == NULL) {
 			*size -= extra_size;
