@@ -28,6 +28,7 @@
 #include "libssl.h"
 #endif
 #include "libfs.h"
+#include "libip.h"
 #include "polarssl/md5.h"
 
 #define RPROXY_ID_LEN 10 /* Must be smaller than 32 */
@@ -120,7 +121,23 @@ t_rproxy *rproxy_setting(char *line) {
 
 	/* Port
 	 */
-	if ((port = strchr(line, ':')) != NULL) {
+#ifdef ENABLE_IPV6
+	if (*line == '[') {
+		line++;
+		if ((port = strchr(line, ']')) == NULL) {
+			return NULL;
+		}
+		*(port++) = '\0';
+		if (*port == '\0') {
+			port = NULL;
+		} else if (*port != ':') {
+			return NULL;
+		}
+	} else
+#endif
+		port = strchr(line, ':');
+
+	if (port != NULL) {
 		*(port++) = '\0';
 		if ((rproxy->port = str2int(port)) < 1) {
 			return NULL;
@@ -128,20 +145,30 @@ t_rproxy *rproxy_setting(char *line) {
 			return NULL;
 		}
 	} else {
-		rproxy->port = 
 #ifdef ENABLE_SSL
-			(rproxy->use_ssl) ? 443 : 
+		if (rproxy->use_ssl) {
+			rproxy->port = 443;
+		} else
 #endif
-			80;
+			rproxy->port = 80;
 	}
 
 	/* Hostname
 	 */
-	if ((rproxy->hostname = strdup(line)) == NULL) {
-		return NULL;
+	if (parse_ip(line, &(rproxy->ip_addr)) == -1) {
+		if ((rproxy->hostname = strdup(line)) == NULL) {
+			return NULL;
+		}
+		rproxy->hostname_len = strlen(rproxy->hostname);
+
+		if (hostname_to_ip(line, &(rproxy->ip_addr)) == -1) {
+			fprintf(stderr, "Can't resolve hostname '%s'\n", line);
+			return NULL;
+		}
+	} else {
+		rproxy->hostname = NULL;
+		rproxy->hostname_len = -1;
 	}
-	rproxy->hostname_len = strlen(rproxy->hostname);
-	hostname_to_ip(line, &(rproxy->ip_addr));
 
 	return rproxy;
 }
@@ -279,18 +306,24 @@ int send_request_to_webserver(t_rproxy_webserver *webserver, t_rproxy_options *o
 
 	/* Send HTTP headers
 	 */
-	if (send_to_webserver(webserver, "Host: ", 6) == -1) {
-		return -1;
-	} else if (send_to_webserver(webserver, rproxy->hostname, rproxy->hostname_len) == -1) {
-		return -1;
-	} else if (send_to_webserver(webserver, rproxy_header, rproxy_header_len) == -1) {
-		return -1;
+	if (rproxy->hostname != NULL) {
+		if (send_to_webserver(webserver, "Host: ", 6) == -1) {
+			return -1;
+		} else if (send_to_webserver(webserver, rproxy->hostname, rproxy->hostname_len) == -1) {
+			return -1;
+		} else if (send_to_webserver(webserver, rproxy_header, rproxy_header_len) == -1) {
+			return -1;
+		}
 	}
 
 	for (headerfield = options->headerfields; headerfield != NULL; headerfield = headerfield->next) {
-		if (strncasecmp(headerfield->data, "Host:", 5) == 0) {
-			continue;
-		} else if (strncasecmp(headerfield->data, "Connection:", 11) == 0) {
+		if (rproxy->hostname != NULL) {
+			if (strncasecmp(headerfield->data, "Host:", 5) == 0) {
+				continue;
+			}
+		}
+
+		if (strncasecmp(headerfield->data, "Connection:", 11) == 0) {
 			continue;
 		}
 
