@@ -16,7 +16,12 @@
 #include <time.h>
 #include <string.h>
 #include <sys/socket.h>
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
 #include <pthread.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -25,11 +30,12 @@
 #endif
 #include "libfs.h"
 #include "libstr.h"
+#include "liblist.h"
 #include "http.h"
 #include "log.h"
 #include "send.h"
-#ifdef ENABLE_COMMAND
-#include "command.h"
+#ifdef ENABLE_TOMAHAWK
+#include "tomahawk.h"
 #endif
 
 #define MAX_CHUNK_SIZE 2048
@@ -55,6 +61,28 @@ char *hs_eol     = "\r\n";                       /*  2 */
 
 static char *unknown_code = "Unknown Error";
 
+static char *ec_doctype = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n";
+static char *ec_head    = "<html>\n<head>\n<title>";
+static char *ec_body1   = "</title>\n<style type=\"text/css\">\n"
+                          "body { background-color:#d0d0d0; font-family:sans-serif }\n"
+                          "div { background-color:#f8f8f8; letter-spacing:4px; width:500px; margin:100px auto 0; padding:50px; "
+                                "border-radius:10px; border:1px solid #808080; box-shadow:8px 15px 20px #404040 }\n"
+                          "h1 { margin:0; font-size:22px; font-weight:normal }\n"
+                          "p { margin:10px 0 0 0; padding-top:2px; font-size:14px; color:#606060; border-top:1px solid #a0a0ff; "
+                              "text-align:right; font-weight:bold }\n"
+                          "</style>\n</head>\n<body>\n<div>\n<h1>";
+static char *ec_body2   = "</h1>\n<p>";
+static char *ec_tail    = "</p>\n</div>\n</body>\n</html>";
+static int ec_doctype_len, ec_head_len, ec_body1_len, ec_body2_len, ec_tail_len;
+
+void init_send_module(void) {
+	ec_doctype_len = strlen(ec_doctype);
+	ec_head_len    = strlen(ec_head);
+	ec_body1_len   = strlen(ec_body1);
+	ec_body2_len   = strlen(ec_body2);
+	ec_tail_len    = strlen(ec_tail);
+}
+
 /* Send a char buffer to the client. Traffic throttling is handled here.
  */
 static int send_to_client(t_session *session, const char *buffer, int size) {
@@ -126,7 +154,7 @@ static int send_to_client(t_session *session, const char *buffer, int size) {
 			session->bytecounter += bytes_sent;
 		}
 	} while (total_sent < size);
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 	increment_transfer(TRANSFER_SEND, total_sent);
 #endif
 
@@ -177,6 +205,7 @@ int send_header(t_session *session) {
 	const char *emesg;
 	time_t t;
 	struct tm *s;
+	t_keyvalue *header;
 
 	/* Send HTTP header.
 	 */
@@ -303,6 +332,23 @@ int send_header(t_session *session) {
 		}
 	}
 
+	/* Custom headers
+	 */
+	header = session->host->custom_headers;
+	while (header != NULL) {
+		if (send_buffer(session, header->key, strlen(header->key)) == -1) {
+			return -1;
+		} else if (send_buffer(session, ": ", 2) == -1) {
+			return -1;
+		} else if (send_buffer(session, header->value, strlen(header->value)) == -1) {
+			return -1;
+		} else if (send_buffer(session, "\r\n", 2) == -1) {
+			return -1;
+		}
+
+		header = header->next;
+	}
+
 	return 0;
 }
 
@@ -377,6 +423,7 @@ int send_code(t_session *session) {
 	int default_port;
 	char ecode[5], len[10], port[10];
 	const char *emesg;
+	size_t ecode_len, emesg_len;
 
 	if (session->return_code == -1) {
 		session->return_code = 500;
@@ -474,12 +521,14 @@ int send_code(t_session *session) {
 
 	ecode[4] = '\0';
 	snprintf(ecode, 4, "%d", session->return_code);
+	ecode_len = strlen(ecode);
 
 	if ((emesg = http_error(session->return_code)) == NULL) {
 		emesg = unknown_code;
 	}
+	emesg_len = strlen(emesg);
 	len[9] = '\0';
-	snprintf(len, 9, "%d", 2 * (int)strlen(emesg) + 380);
+	snprintf(len, 9, "%d", (int)((2 * emesg_len) + (2 * ecode_len) + 3 + ec_doctype_len + ec_head_len + ec_body1_len + ec_body2_len + ec_tail_len));
 
 	if (send_buffer(session, "Content-Length: ", 16) == -1) {
 		return -1;
@@ -498,30 +547,25 @@ int send_code(t_session *session) {
 		return 0;
 	}
 
-	if (send_buffer(session, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n", 91) == -1) {
+	if (send_buffer(session, ec_doctype, ec_doctype_len) == -1) {
 		return -1;
-	} else if (send_buffer(session, "<html>\n<head>\n<title>", 21) == -1) {
+	} else if (send_buffer(session, ec_head, ec_head_len) == -1) {
 		return -1;
-	} else if (send_buffer(session, ecode, 3) == -1) {
+	} else if (send_buffer(session, ecode, ecode_len) == -1) {
 		return -1;
 	} else if (send_buffer(session, " - ", 3) == -1) {
 		return -1;
-	} else if (send_buffer(session, emesg, strlen(emesg)) == -1) {
+	} else if (send_buffer(session, emesg, emesg_len) == -1) {
 		return -1;
-	} else if (send_buffer(session,	"</title>\n<style type=\"text/css\">" /* 32 */
-	                                "BODY { color:#ffffff ; background-color:#00000a }\n" /* 50 */
-	                                "DIV { font-family:sans-serif ; font-size:30px ; letter-spacing:20px ; " /* 70 */
-	                                "text-align:center ; position:relative ; top:250px }\n" /* 52 */
-	                                "</style>\n</head>\n" /* 17 */
-	                                "<body>\n<div>", 233) == -1) { /* 12 */
+	} else if (send_buffer(session,	ec_body1, ec_body1_len) == -1) {
 		return -1;
-	} else if (send_buffer(session, ecode, 3) == -1) {
+	} else if (send_buffer(session, emesg, emesg_len) == -1) {
 		return -1;
-	} else if (send_buffer(session, " - ", 3) == -1) {
+	} else if (send_buffer(session,	ec_body2, ec_body2_len) == -1) {
 		return -1;
-	} else if (send_buffer(session, emesg, strlen(emesg)) == -1) {
+	} else if (send_buffer(session, ecode, ecode_len) == -1) {
 		return -1;
-	} else if (send_buffer(session, "</div>\n</body>\n</html>\n", 23) == -1) {
+	} else if (send_buffer(session, ec_tail, ec_tail_len) == -1) {
 		return -1;
 	}
 
