@@ -17,7 +17,7 @@
 #include <string.h>
 #include <pthread.h>
 #include <unistd.h>
-#ifdef ENABLE_COMMAND
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
 #ifdef ENABLE_SSL
@@ -42,7 +42,9 @@ enum t_section { syntax_error = -1, none, binding, virtual_host, directory, fcgi
 
 static bool including = false;
 static t_keyvalue *variables = NULL;
+#ifdef ENABLE_XSLT
 static char *index_xslt = CONFIG_DIR"/index.xslt";
+#endif
 
 static t_host *new_host(void) {
 	t_host *host;
@@ -82,6 +84,7 @@ static t_host *new_host(void) {
 	init_charlist(&(host->required_binding));
 	init_charlist(&(host->required_group));
 	init_charlist(&(host->alter_group));
+	host->custom_headers     = NULL;
 #ifdef ENABLE_TOOLKIT
 	init_charlist(&(host->toolkit_rules));
 #endif
@@ -203,7 +206,6 @@ static t_binding *new_binding(void) {
 	binding->ca_crl               = NULL;
 #endif
 	binding->binding_id           = NULL;
-	binding->socket               = -1;
 	binding->enable_trace         = false;
 	binding->enable_alter         = false;
 	binding->max_keepalive        = 50;
@@ -212,6 +214,9 @@ static t_binding *new_binding(void) {
 	binding->time_for_1st_request = 5;
 	binding->time_for_request     = 30;
 
+	binding->socket               = -1;
+	binding->poll_data            = NULL;
+
 	binding->next                 = NULL;
 
 	return binding;
@@ -246,8 +251,8 @@ t_config *default_config(void) {
 	config->mimetype_config    = "mimetype.conf";
 
 	config->binding            = NULL;
-#ifdef ENABLE_COMMAND
-	config->command_port       = NULL;
+#ifdef ENABLE_TOMAHAWK
+	config->tomahawk_port      = NULL;
 #endif
 
 #ifdef ENABLE_CHROOT
@@ -320,8 +325,8 @@ t_config *default_config(void) {
 	config->cache_min_filesize = 1;
 #endif
 
-#ifdef ENABLE_COMMAND
-	config->command_port       = NULL;
+#ifdef ENABLE_TOMAHAWK
+	config->tomahawk_port      = NULL;
 #endif
 
 #ifdef ENABLE_MONITOR
@@ -658,7 +663,7 @@ static bool system_setting(char *key, char *value, t_config *config) {
 	t_cgi_handler *cgi;
 	t_throttle *throt;
 	int speed;
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 	t_binding *binding;
 	char *port, *password;
 #endif
@@ -730,24 +735,6 @@ static bool system_setting(char *key, char *value, t_config *config) {
 				return true;
 			}
 		}
-#endif
-#ifdef ENABLE_COMMAND
-	} else if (strcmp(key, "commandchannel") == 0) {
-		if (split_string(value, &port, &password, ',') == 0) {
-			binding = new_binding();
-			set_to_localhost(&(binding->interface));
-
-			binding->next = config->command_port;
-			config->command_port = binding;
-
-			if ((config->command_port->port = str2int(port)) > 0) {
-				if ((config->command_port->binding_id = strdup(password)) != NULL) {
-					return true;
-				}
-			} else {
-				free(config->command_port);
-			}
-		}
 #endif
 	} else if (strcmp(key, "cgiextension") == 0) {
 #ifdef CIFS
@@ -877,7 +864,7 @@ static bool system_setting(char *key, char *value, t_config *config) {
 		free(alist);
 
 #ifdef ENABLE_XSLT
-		if ((monitor_host->show_index = strdup("xml")) == false) {
+		if ((monitor_host->show_index = strdup("xml")) == NULL) {
 			return false;
 		}
 #endif
@@ -984,6 +971,24 @@ static bool system_setting(char *key, char *value, t_config *config) {
 				}
 			}
 		}
+#ifdef ENABLE_TOMAHAWK
+	} else if (strcmp(key, "tomahawk") == 0) {
+		if (split_string(value, &port, &password, ',') == 0) {
+			binding = new_binding();
+			set_to_localhost(&(binding->interface));
+
+			binding->next = config->tomahawk_port;
+			config->tomahawk_port = binding;
+
+			if ((config->tomahawk_port->port = str2int(port)) > 0) {
+				if ((config->tomahawk_port->binding_id = strdup(password)) != NULL) {
+					return true;
+				}
+			} else {
+				free(config->tomahawk_port);
+			}
+		}
+#endif
 	} else if (strcmp(key, "userdirectory") == 0) {
 		if ((*value != '/') && (strchr(value, '.') == NULL)) {
 			if ((config->user_directory = strdup(value)) != NULL) {
@@ -1162,6 +1167,10 @@ static bool host_setting(char *key, char *value, t_host *host) {
 		if (parse_yesno(value, &(host->allow_dot_files)) == 0) {
 			return true;
 		}
+	} else if (strcmp(key, "customheader") == 0) {
+		if (parse_keyvalue(value, &(host->custom_headers), ":") != -1) {
+			return true;
+		}
 	} else if (strcmp(key, "denybody") == 0) {
 		if (host->deny_body == NULL) {
 			host->deny_body = (t_deny_body*)malloc(sizeof(t_deny_body));
