@@ -14,6 +14,7 @@
 
 #include <stdbool.h>
 #include <pthread.h>
+#include <poll.h>
 #include <regex.h>
 #include "global.h"
 #include "libip.h"
@@ -130,6 +131,7 @@ typedef struct type_binding {
 	int           time_for_request;
 
 	int           socket;
+	struct pollfd *poll_data;
 
 	struct type_binding *next;
 } t_binding;
@@ -203,6 +205,7 @@ typedef struct type_host {
 	t_denybotlist   *deny_bot;
 	t_charlist      required_group;
 	t_charlist      alter_group;
+	t_keyvalue      *custom_headers;
 	char            *wrap_cgi;
 	t_groups        groups;
 	t_charlist      volatile_object;
@@ -321,8 +324,8 @@ typedef struct type_config {
 	off_t         cache_min_filesize;
 #endif
 
-#ifdef ENABLE_COMMAND
-	t_binding     *command_port;
+#ifdef ENABLE_TOMAHAWK
+	t_binding     *tomahawk_port;
 #endif
 
 #ifdef ENABLE_MONITOR
