@@ -18,6 +18,7 @@
 #include <pthread.h>
 #include <pwd.h>
 #include <sys/socket.h>
+#include "global.h"
 #include "alternative.h"
 #include "libstr.h"
 #include "libip.h"
@@ -27,8 +28,8 @@
 #ifdef ENABLE_MONITOR
 #include "monitor.h"
 #endif
-#ifdef ENABLE_COMMAND
-#include "command.h"
+#ifdef ENABLE_TOMAHAWK
+#include "tomahawk.h"
 #endif
 #include <regex.h>
 
@@ -111,7 +112,7 @@ static void clear_session(t_session *session) {
 /* Initialize a session-record.
  */
 void init_session(t_session *session) {
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 	increment_counter(COUNTER_CLIENT);
 #endif
 	if ((session->client_id = new_client_id++) == MAX_CHILD_ID) {
@@ -495,9 +496,9 @@ bool client_is_rejected_bot(t_session *session) {
 /* Remove port from hostname
  */
 int remove_port_from_hostname(char *hostname, t_binding *binding) {
-	char *c, old;
+	char *c;
 #ifdef ENABLE_IPV6
-	char ip[IPv6_LEN];
+	char old, ip[IPv6_LEN];
 #endif
 
 	if (hostname == NULL) {
@@ -565,7 +566,7 @@ int prevent_xss(t_session *session) {
 
 	if (result > 0) {
 		log_exploit_attempt(session, "XSS", session->vars);
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 		increment_counter(COUNTER_EXPLOIT);
 #endif
 #ifdef ENABLE_MONITOR
@@ -622,7 +623,7 @@ static int prevent_sqli_str(t_session *session, char *str, int length) {
 	while (pattern != NULL) {
 		if (regexec(&(pattern->regex), data, 0, NULL, 0) != REG_NOMATCH) {
 			log_exploit_attempt(session, "SQLi", str);
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 			increment_counter(COUNTER_EXPLOIT);
 #endif
 #ifdef ENABLE_MONITOR
@@ -738,7 +739,7 @@ int prevent_csrf(t_session *session) {
 
 	if (session->body != NULL) {
 		log_exploit_attempt(session, "CSRF", session->body);
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 		increment_counter(COUNTER_EXPLOIT);
 #endif
 #ifdef ENABLE_MONITOR
