@@ -18,6 +18,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
+#include <poll.h>
 #include <time.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -138,12 +139,11 @@ void print_error(t_config *config, char *message) {
 /* Execute file
  */
 int execute_command(char *command, t_exec_type type) {
-	int output_pipe[2], error_pipe[2], result, skip;
-	int bytes_read, bytes_in_buffer = 0, highest_fd, open_pipes = 2;
+	int output_pipe[2], error_pipe[2], result, skip, poll_size;
+	int bytes_read, bytes_in_buffer = 0, open_pipes = 2;
 	char buffer[CGI_BUFFER_SIZE], *pos;
 	bool header_read;
-	struct timeval select_timeout;
-	fd_set read_fds;
+	struct pollfd poll_data[2];
 	pid_t cgi_pid;
 
 	fflush(stdout);
@@ -159,138 +159,128 @@ int execute_command(char *command, t_exec_type type) {
 		return -1;
 	}
 
-	switch (cgi_pid = fork()) {
-		case -1:
-			break;
-		case 0:
-			/* Child. Executes command.
-			 */
-			dup2(output_pipe[1], STDOUT_FILENO);
-			dup2(error_pipe[1], STDERR_FILENO);
-
-			close(output_pipe[0]);
-			close(output_pipe[1]);
-			close(error_pipe[0]);
-			close(error_pipe[1]);
-
-			fcntl(STDOUT_FILENO, F_SETFD, 0);
-			fcntl(STDERR_FILENO, F_SETFD, 0);
-
-			if (type == cgi) {
-				if ((pos = strchr(command, '?')) != NULL) {
-					*(pos++) = '\0';
-					setenv("QUERY_STRING", pos, 1);
-				}
-				url_decode(command);
-
-				execlp(command, command, (char*)NULL);
-			} else {
-				if ((result = system(command)) != -1) {
-					exit(result);
-				}
-			}
-
-			exit(EXIT_FAILURE);
-		default:
-			/* Parent. Reads command output.
-			 */
-			close(output_pipe[1]);
-			close(error_pipe[1]);
-
-			do {
-				highest_fd = -1;
-
-				FD_ZERO(&read_fds);
-				if (output_pipe[0] != -1) {
-					FD_SET(output_pipe[0], &read_fds);
-					if (output_pipe[0] > highest_fd) {
-						highest_fd = output_pipe[0];
-					}
-				}
-				if (error_pipe[0] != -1) {
-					FD_SET(error_pipe[0], &read_fds);
-					if (error_pipe[0] > highest_fd) {
-						highest_fd = error_pipe[0];
-					}
-				}
-
-				if (highest_fd == -1) {
-					break;
-				}
-
-				select_timeout.tv_sec = 1;
-				select_timeout.tv_usec = 0;
-
-				switch (select(highest_fd + 1, &read_fds, NULL, NULL, &select_timeout)) {
-					case -1:
-						return -1;
-					case 0:
-						break;
-					default:
-						/* Output
-						 */
-						if (output_pipe[0] != -1) {
-							if (FD_ISSET(output_pipe[0], &read_fds)) {
-								bytes_read = read(output_pipe[0], buffer + bytes_in_buffer, CGI_BUFFER_SIZE - bytes_in_buffer);
-								switch (bytes_read) {
-									case -1:
-										return -1;
-									case 0:
-										close_pipe_end(&output_pipe[0], &open_pipes);
-										header_read = true;
-										break;
-									default:
-										if (header_read == false) {
-											bytes_in_buffer += bytes_read;
-
-											if ((pos = strstr(buffer, "\r\n\r\n")) == NULL) {
-												pos = strstr(buffer, "\n\n");
-												skip = 2;
-											} else {
-												skip = 4;
-											}
-
-											if (pos != NULL) {
-												/* Skip CGI header
-												 */
-												pos += skip;
-												if (write_buffer(STDOUT_FILENO, pos, bytes_in_buffer - (pos - buffer)) == -1) {
-													return -1;
-												}
-												header_read = true;
-												bytes_in_buffer = 0;
-											}
-										} else if (write_buffer(STDOUT_FILENO, buffer, bytes_read) == -1) {
-											return -1;
-										}
-								}
-							}
-						}
-
-						/* Error
-						 */
-						if ((error_pipe[0] != -1) && header_read) {
-							if (FD_ISSET(error_pipe[0], &read_fds)) {
-								bytes_read = read(error_pipe[0], buffer, CGI_BUFFER_SIZE);
-								switch (bytes_read) {
-									case -1:
-										return -1;
-									case 0:
-										close_pipe_end(&error_pipe[0], &open_pipes);
-										break;
-									default:
-										if (write_buffer(STDERR_FILENO, buffer, bytes_read) == -1) {
-											return -1;
-										}
-								}
-							}
-						}
-				}
-			} while (open_pipes > 0);
-
-			waitpid(cgi_pid, NULL, 0);
+	if ((cgi_pid = fork()) == -1) {
+		return -1;
 	}
 
+	if (cgi_pid == 0) {
+		/* Child. Executes command.
+		 */
+		dup2(output_pipe[1], STDOUT_FILENO);
+		dup2(error_pipe[1], STDERR_FILENO);
+
+		close(output_pipe[0]);
+		close(output_pipe[1]);
+		close(error_pipe[0]);
+		close(error_pipe[1]);
+
+		fcntl(STDOUT_FILENO, F_SETFD, 0);
+		fcntl(STDERR_FILENO, F_SETFD, 0);
+
+		if (type == cgi) {
+			if ((pos = strchr(command, '?')) != NULL) {
+				*(pos++) = '\0';
+				setenv("QUERY_STRING", pos, 1);
+			}
+			url_decode(command);
+
+			execlp(command, command, (char*)NULL);
+		} else {
+			if ((result = system(command)) != -1) {
+				exit(result);
+			}
+		}
+
+		exit(EXIT_FAILURE);
+	}
+
+	/* Parent. Reads command output.
+	 */
+	close(output_pipe[1]);
+	close(error_pipe[1]);
+
+	poll_size = 0;
+	if (output_pipe[0] != -1) {
+		poll_data[poll_size].fd = output_pipe[0];
+		poll_data[poll_size].events = POLL_EVENT_BITS;
+		poll_size++;
+	}
+	if (error_pipe[0] != -1) {
+		poll_data[poll_size].fd = error_pipe[0];
+		poll_data[poll_size].events = POLL_EVENT_BITS;
+		poll_size++;
+	}
+
+	if (poll_size == 0) {
+		return -1;
+	}
+
+	do {
+		switch (poll((struct pollfd*)&poll_data, poll_size, 1000)) {
+			case -1:
+				return -1;
+			case 0:
+				break;
+			default:
+				/* Output
+				 */
+				if ((output_pipe[0] != -1) && (poll_data[0].revents != 0)) {
+					bytes_read = read(output_pipe[0], buffer + bytes_in_buffer, CGI_BUFFER_SIZE - bytes_in_buffer);
+					switch (bytes_read) {
+						case -1:
+							return -1;
+						case 0:
+							close_pipe_end(&output_pipe[0], &open_pipes);
+							header_read = true;
+							break;
+						default:
+							if (header_read == false) {
+								bytes_in_buffer += bytes_read;
+
+								if ((pos = strstr(buffer, "\r\n\r\n")) == NULL) {
+									pos = strstr(buffer, "\n\n");
+									skip = 2;
+								} else {
+									skip = 4;
+								}
+
+								if (pos != NULL) {
+									/* Skip CGI header
+									 */
+									pos += skip;
+									if (write_buffer(STDOUT_FILENO, pos, bytes_in_buffer - (pos - buffer)) == -1) {
+										return -1;
+									}
+									header_read = true;
+									bytes_in_buffer = 0;
+								}
+							} else if (write_buffer(STDOUT_FILENO, buffer, bytes_read) == -1) {
+								return -1;
+							}
+					}
+				}
+
+				/* Error
+				 */
+				if ((error_pipe[0] != -1) && header_read && (poll_data[1].revents != 0)) {
+					bytes_read = read(error_pipe[0], buffer, CGI_BUFFER_SIZE);
+					switch (bytes_read) {
+						case -1:
+							return -1;
+						case 0:
+							close_pipe_end(&error_pipe[0], &open_pipes);
+							break;
+						default:
+							if (write_buffer(STDERR_FILENO, buffer, bytes_read) == -1) {
+								return -1;
+							}
+					}
+				}
+		}
+	} while (open_pipes > 0);
+
+	waitpid(cgi_pid, NULL, 0);
+
 	return 0;
 }
 
