@@ -38,8 +38,8 @@
 #ifdef ENABLE_CACHE
 #include "cache.h"
 #endif
-#ifdef ENABLE_COMMAND
-#include "command.h"
+#ifdef ENABLE_TOMAHAWK
+#include "tomahawk.h"
 #endif
 #ifdef ENABLE_XSLT
 #include "xslt.h"
@@ -86,7 +86,7 @@ int send_file(t_session *session) {
 	t_cached_object *cached_object;
 #endif
 
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 	increment_counter(COUNTER_FILE);
 #endif
 	session->mimetype = get_mimetype(session->extension, session->config->mimetype);
@@ -467,7 +467,7 @@ int execute_cgi(t_session *session) {
 	t_cgi_info cgi_info;
 	pid_t cgi_pid = -1;
 
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 	increment_counter(COUNTER_CGI);
 #endif
 
@@ -702,7 +702,7 @@ int execute_cgi(t_session *session) {
 						/* Add body content to cache buffer
 						 */
 						if ((cache_buffer != NULL) && (retval == 200)) {
-							if (cache_size + cgi_info.input_len > session->config->cache_max_filesize) {
+							if ((unsigned int)(cache_size + cgi_info.input_len) > session->config->cache_max_filesize) {
 								free(cache_buffer);
 								cache_buffer = NULL;
 							} else {
@@ -904,7 +904,7 @@ int execute_cgi(t_session *session) {
 								if (retval != 200) {
 									free(cache_buffer);
 									cache_buffer = NULL;
-								} else if (cache_size + cgi_info.input_len > session->config->cache_max_filesize) {
+								} else if ((unsigned int)(cache_size + cgi_info.input_len) > session->config->cache_max_filesize) {
 									free(cache_buffer);
 									cache_buffer = NULL;
 								} else {
@@ -1087,7 +1087,7 @@ static t_access allow_alter(t_session *session) {
 /* Handle PUT requests
  */
 int handle_put_request(t_session *session) {
-	int handle_write, handle_read = -1, result = -1, total_written = 0, lock_timeout;
+	int auth_result, handle_write, handle_read = -1, result = -1, total_written = 0, lock_timeout;
 	off_t write_begin, write_end, total_size, file_size;
 	ssize_t bytes_read;
 	char *range, *value, *rest, *buffer;
@@ -1108,8 +1108,8 @@ int handle_put_request(t_session *session) {
 		case allow:
 			break;
 		case pwd:
-			if (http_authentication_oke(session, false) == false) {
-				return 401;
+			if ((auth_result = http_authentication_result(session, false)) != 200) {
+				return auth_result;
 			}
 			if (group_oke(session, session->remote_user, &(session->host->alter_group)) == false) {
 				return 403;
@@ -1179,9 +1179,9 @@ int handle_put_request(t_session *session) {
 		if (strncmp(range, "bytes ", 6) != 0) {
 			result = 416;
 		} else {
-			range = strdup(range + 6);
-
-			if (split_string(range, &value, &rest, '-') == -1) {
+			if ((range = strdup(range + 6)) == NULL) {
+				result = -1;
+			} else if (split_string(range, &value, &rest, '-') == -1) {
 				result = 416;
 			} else if (strlen(value) > 9) {
 				result = 416;
@@ -1273,6 +1273,8 @@ int handle_put_request(t_session *session) {
 /* Handle DELETE requests
  */
 int handle_delete_request(t_session *session) {
+	int auth_result;
+
 	/* Access check
 	 */
 	switch (allow_alter(session)) {
@@ -1283,8 +1285,8 @@ int handle_delete_request(t_session *session) {
 		case allow:
 			break;
 		case pwd:
-			if (http_authentication_oke(session, false) == false) {
-				return 401;
+			if ((auth_result = http_authentication_result(session, false)) != 200) {
+				return auth_result;
 			}
 			if (group_oke(session, session->remote_user, &(session->host->alter_group)) == false) {
 				return 403;
