@@ -465,7 +465,10 @@ static int do_rewrite(char *url, regex_t *regexp, regmatch_t *pmatch, char *rep,
 }
 
 void init_toolkit_options(t_toolkit_options *options, char *website_root, t_url_toolkit *toolkit,
-	                      bool use_ssl, bool allow_dot_files, t_headerfield *headerfields) {
+#ifdef ENABLE_SSL
+	                      bool use_ssl, 
+#endif
+						  bool allow_dot_files, t_headerfield *headerfields) {
 	options->sub_depth = 0;
 	options->new_url = NULL;
 	options->website_root = website_root;
@@ -473,7 +476,9 @@ void init_toolkit_options(t_toolkit_options *options, char *website_root, t_url_
 	options->ban = 0;
 	options->expire = -1;
 	options->url_toolkit = toolkit;
+#ifdef ENABLE_SSL
 	options->use_ssl = use_ssl;
+#endif
 	options->allow_dot_files = allow_dot_files;
 	options->headerfields = headerfields;
 }
