@@ -31,7 +31,11 @@
 #define IU_ISFILE        1
 #define IU_ISDIR         2
 
-typedef enum { tc_none, tc_match, tc_requesturi, tc_usessl, tc_oldbrowser } t_toolkit_condition;
+typedef enum { tc_none, tc_match, tc_requesturi, 
+#ifdef ENABLE_SSL
+               tc_usessl, 
+#endif
+               tc_oldbrowser } t_toolkit_condition;
 typedef enum { to_none, to_rewrite, to_sub, to_expire, to_skip, to_denyaccess, to_redirect,
 	to_fastcgi, to_ban, to_replace } t_toolkit_operation;
 typedef enum { tf_continue, tf_return, tf_exit } t_toolkit_flow;
@@ -64,7 +68,9 @@ typedef struct type_toolkit_options {
 	char *fastcgi_server;
 	int  ban;
 	int  expire;
+#ifdef ENABLE_SSL
 	bool use_ssl;
+#endif
 	bool allow_dot_files;
 	t_url_toolkit *url_toolkit;
 	t_headerfield *headerfields;
@@ -74,7 +80,10 @@ t_url_toolkit *find_toolkit(char *toolkit_id, t_url_toolkit *url_toolkit);
 bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit);
 bool toolkit_rules_oke(t_url_toolkit *url_toolkit);
 void init_toolkit_options(t_toolkit_options *options, char *website_root, t_url_toolkit *toolkit,
-                          bool use_ssl, bool allow_dot_files, t_headerfield *headerfields);
+#ifdef ENABLE_SSL
+                          bool use_ssl, 
+#endif
+                          bool allow_dot_files, t_headerfield *headerfields);
 int use_toolkit(char *url, char *toolkit_id, t_toolkit_options *options);
 
 #endif
