@@ -27,6 +27,7 @@
 #include "alternative.h"
 #include "libstr.h"
 #include "libfs.h"
+#include "libip.h"
 #ifdef ENABLE_TOOLKIT
 #include "toolkit.h"
 #endif
@@ -193,7 +194,9 @@ int read_file(char *config_file, t_line **config, t_line **aliases, bool handle_
 		printf("Reading %s\n", config_file);
 	}
 
-	if ((fp = fopen(config_file, "r")) == NULL) {
+	if (config_file == NULL) {
+		return -1;
+	} else if ((fp = fopen(config_file, "r")) == NULL) {
 		perror(config_file);
 		return -1;
 	}
@@ -282,19 +285,9 @@ int read_config_file(char *config_file, t_line **config) {
 }
 
 bool is_ip_address(char *str) {
-	char *digit;
-	int digits = 0, value;
+	t_ip_addr ip_addr;
 
-	while (str != NULL) {
-		split_string(str, &digit, &str, '.');
-		value = str2int(digit);
-		if ((value < 0) || (value > 255)) {
-			return false;
-		}
-		digits++;
-	}
-
-	return (digits == 4);
+	return parse_ip(str, &ip_addr) != -1;
 }
 
 int find_duplicates(t_line *config, char *key, char *config_dir) {
@@ -394,7 +387,7 @@ int check_main_config(char *config_dir) {
 	haystack = search_key(config, "fastcgiid");
 	needles = needle = search_key(config, "usefastcgi");
 	while (needle != NULL) {
-		if ((rest = strdup(needle->value)) == false) {
+		if ((rest = strdup(needle->value)) == NULL) {
 			mem_error();
 		}
 		while (rest != NULL) {
@@ -660,7 +653,11 @@ void check_url_toolkit(char *config_dir, char **toolkit_id) {
 			printf("Bad URL: missing leading slash.\n");
 		}
 
+#ifdef ENABLE_SSL
 		init_toolkit_options(&options, ".", url_toolkit, false, false, NULL);
+#else
+		init_toolkit_options(&options, ".", url_toolkit, false, NULL);
+#endif
 
 		id = toolkit_id;
 		while (*id != NULL) {
