@@ -25,8 +25,8 @@
 #include "libstr.h"
 #include "send.h"
 #include "log.h"
-#ifdef ENABLE_COMMAND
-#include "command.h"
+#ifdef ENABLE_TOMAHAWK
+#include "tomahawk.h"
 #endif
 
 #define XSLT_INDEX "/index.xslt\0"
@@ -487,7 +487,7 @@ int show_index(t_session *session) {
 	char line[LINE_SIZE + 1];
 */
 
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 	increment_counter(COUNTER_INDEX);
 #endif
 
